#!/bin/bash

function usage {
  [[ ${1:-} = "-s" ]] || echo "Скрипт предназначен для получения образа или файла с данными из папки $installdir/images или сайта https://download.protei-sm.ru"
  echo "Использование: getimage.sh [-p|--pull] filename|image"
  echo "  -p, --pull - принудительно загрузить с сайта"
  echo "  -h, --help - показать справку"
  exit 1
}

# DEBUG options
PROGRAM=${0##*/}
if [[ $PROGRAM == "getimage.sh" ]]; then
  if [[ -n ${DEBUG:-} ]]; then
    exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
    BASH_XTRACEFD="5"
    PS4='$LINENO: '
    set -x
  fi
  set -euo pipefail
fi

workdir=$(dirname $(realpath -s $0))
installdir=$workdir
imagespath=$installdir/images
[[ ! -d $imagespath ]] && mkdir "$imagespath"

if [ -z $(which curl) ] || [ -z $(which bzip2) ] || [ -z $(which sha256sum) ]; then
  echo "Устанавливаем curl bzip2 coreutils"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install curl bzip2 coreutils 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

forcePull=""
while getopts ':-:hp' KEY ; do
  case $KEY in
    p ) forcePull=true ;;
    h ) usage ;;
    - )
        case $OPTARG in
          pull ) forcePull=true ;;
          help ) usage ;;
          * )
              echo "Не верная опция $OPTARG при вызове $PROGRAM"
              usage -s
          ;;
        esac
    ;;
    * ) 
        echo "Не верная опция $OPTARG при вызове $PROGRAM"
        usage -s
    ;;
  esac
done
shift $((OPTIND -1))

[ $# -gt 0 ] || { echo "Не указан необходимый образ"; exit 1; }
[[ -z $1 && -n ${2:-} ]] && image="$2" || image="$1"

cd "$imagespath" || exit 1

case $image in
  psm/*) imageType="psm" ;;
  *.tar.gz) imageType="tgz" ;;
  *.tgz) imageType="tgz" ;;
  *.pbf) imageType="pbf" ;;
  *.zip) imageType="zip"; imagespath=$installdir/gis/data/extdata; cd $imagespath ;;
  *) imageType="other" ;;  
esac
echo "Получаем файл с $image"
[[ $imageType =~ (tgz|pbf|zip) ]] && imageFile=$image || imageFile=$(echo $image | sed 's/\//_/;s/:/__/').bz2
fullpath=$imagespath/$imageFile

if [ -z $forcePull ] && [ -f "$fullpath" ] ; then
  echo "Будет использован $image уже сохранённый в $fullpath"
  if [[ ! $imageType =~ (tgz|pbf|zip) ]]; then
    if [[ $(docker image ls -q $image | wc -l) -eq 0 ]]; then
      echo "Импортируем образ $image в docker..."
      if ! docker load -i $fullpath; then 
        echo "Не удалось загрузить образ из $imageFile в docker"
        success=1
      fi
    fi
  elif [[ $imageType == tgz ]]; then
    echo -n "Проверяем файл архива..."
    tar -tf "$fullpath" &> /dev/null && { echo "OK"; success=0; } || { echo "FAIL"; success=1; }
  fi
  [[ ${success:-0} -eq 0 ]] && exit 0
fi
while [[ ${success:-1} -eq 1 ]]; do
  echo "Получаем $image с сайта download.protei-sm.ru"
  [[ $imageType =~ (pbf|zip) ]] && downlPath="https://download.protei-sm.ru/maps/$imageFile" || downlPath="https://download.protei-sm.ru/images/$imageFile"
  if ! curl --http1.1 -f -o /dev/null --silent --head "$downlPath" &> /dev/null; then
    echo "Не удалось проверить наличие файла по адресу $downlPath"
    success=1
  else
    if curl --http1.1 -O -f -# --retry 3 --retry-delay 5 -Y 1 "$downlPath"; then
      if curl --http1.1 -f -o /dev/null --silent --head "$downlPath".sha256 &> /dev/null; then
        echo -n "Проверяем контрольную сумму..."
        if curl --http1.1 -O -f -s --retry 3 --retry-delay 5 -Y 1 "$downlPath".sha256; then
          sha256sum -c --quiet --status "$imageFile".sha256 && { echo "ОК"; success=0; } || { echo "FAIL"; success=1; }
        else
          { echo "FAIL"; success=1; }
        fi 
      else
        success=0
      fi
      TZ=GMT LANG=C date -r "$imageFile" '+%a, %d %b %Y %T %Z' > "$imageFile".lastmod
    else
      success=1
    fi
  fi
  if [[ ! $imageType =~ (tgz|pbf|zip) ]]; then
    if [[ $success -ne 0 ]]; then
      echo "Ошибка получения"
      if [ $imageType = "other" ]; then
      # если open source
        echo "Получаем образ $image с hub.docker.com..."
        while [[ ${success:-1} -eq 1 ]]; do
          if ! docker pull $image; then
            echo "Не удалось получить образ $image"
            if [[ -f $fullpath ]]; then
              read -r -p "Использовать образ $image из $fullpath ? [Y/n]: " ANSWER
              if [[ "${ANSWER:-y}" =~ ^[yY]$ ]]; then
                success=3
              else
                read -r -p "Повторить попытку получить $image из hub.docker.com? [Y/n]: " ANSWER
                [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit 1
              fi
            else
              break
            fi  
          else
            echo "Сохраняем $image в $fullpath"
            while true;do echo -n .;sleep 1;done &
            docker save $image | bzip2 > $fullpath
            kill $!; trap 'kill $!' SIGTERM
            echo
            exit
          fi
        done
      else
      # если psm
        read -r -p "Повторить попытку получить $image ? [Y/n]: " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || exit 1
      fi
    elif [[ $success -ne 1 ]]; then
      echo "Импортируем образ $image в docker..."
      docker load -i "$imageFile" || { echo "Ошибка импорта"; success=1; }
    fi
  fi
  if [[ $success -ne 0 ]]; then
    read -r -p "Повторить попытку получить $image ? [Y/n]: " ANSWER
    [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit 1
  fi
done
exit 0