#!/bin/bash

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

function usage {
      [[ ${1:-} = "-s" ]] || echo "Скрипт для загрузки или обновления репозиториев в Gitea из файла с расширением .yml или .yaml"
      echo "Использование:"
      echo "  import-repos.sh [-h|--help]"
      echo "  import-repos.sh -f|--file path"
      echo "    path      - относительный или полный путь к файлу с параметрами загружаемых файлов"
      echo "    -h|--help - отобразить справку"
      exit 1
}

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
cd "$workdir" || exit 1
PROGRAM=${0##*/}

[[ -f $installdir/install/functions.sh ]] || { echo "Отсутствует файл $installdir/install/functions.sh"; exit 2; }
source $installdir/install/functions.sh

while getopts ':h-f:' KEY; do
  case $KEY in
    h ) usage ;;
    f ) filepath=${OPTARG} ;;
    - )
        case $OPTARG in
          help ) usage ;;
          file ) 
                shift 
                filepath=${OPTARG} 
                ;;
          : ) 
              echo "Не указан файл для импорта"
              usage -s; exit 1
              ;;
          * )
              echo "Не верная опция $OPTARG при вызове $PROGRAM"
              usage -s; exit 1
              ;;
        esac
        ;;
    : ) 
        echo "Не указан файл для импорта"
        usage -s; exit 1
        ;;
    * )
        echo "Не верная опция $OPTARG при вызове $PROGRAM"
        usage -s; exit 1
        ;;
  esac
done
shift $((OPTIND -1))

[[ -f ${filepath:-} ]] || { echo "Указанный файл отсутствует или к нему нет доступа"; exit 2; }
[[ $filepath =~ \.ya?ml$ ]] || { echo "Расширение указанного фала должно быть .yml или .yaml"; exit 2; }
eval $(parse_yaml $filepath) || { echo "Ошибка формата YAML в указанном файле"; exit 2; }

source "${installdir}/proxy/.env"
source $installdir/gitea/.env

if [ -z $(which curl) ] || [ -z $(which rsync) ] || [ -z $(which git) ]; then
  echo "Устанавливаем curl rsync"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install curl rsync git 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 


# проверяем настройку Gitea
source "$installdir"/gitea/check.sh
check_token_exist

for f in $repo_ ; do
  if ! eval curl -ksf --noproxy '\*' https://git.$SERVICE_DOMAIN/api/v1/orgs/\$${f}_owner?token=$token &>/dev/null; then
    if ! eval curl -ksf --noproxy '\*' -X POST https://git.$SERVICE_DOMAIN/api/v1/orgs?token=$token -H \'accept: application/json\' -H \'Content-Type: application/json\' -d '\{\"full_name\":\"'\$${f}_owner'\",\"repo_admin_change_team_access\":true,\"username\":\"'\$${f}_owner'\",\"visibility\":\"public\",\"website\":\"https://'$SERVICE_DOMAIN'\"\}' &>/dev/null; then
      eval echo "Ошибка создания организации \$${f}_owner"
      exit 2
    fi
  fi
  if ! eval curl -ksf --noproxy '\*' https://git.$SERVICE_DOMAIN/api/v1/repos/\$${f}_owner/\$${f}?token=$token &>/dev/null; then
    description=$(eval echo \$${f}_description | sed 's/ /\\ /g')
    if ! eval curl -ksf --noproxy '\*' -X POST https://git.$SERVICE_DOMAIN/api/v1/orgs/\$${f}_owner/repos?token=$token -H \'accept: application/json\' -H \'Content-Type: application/json\' -d '\{\"name\":\"'\$${f}'\",\"template\":'\$${f}_template',\"auto_init\":true,\"description\":\"'$description'\"\}' &>/dev/null; then
      eval echo "Ошибка создания репозитория \$${f}_owner/\$${f}"
      exit 2
    fi
  fi
  
  if eval [[ \$${f}_download = true ]]; then
    pull_flag='-p'
  else
    pull_flag=''
  fi
  eval "$installdir"/getimage.sh $pull_flag \$${f}_file || exit 1
  mkdir -p "$workdir"/data/distr
  eval tar -xf "$installdir"/images/\$${f}_file -C "$workdir"/data/distr/

  eval echo -n "Обновляем репозиторий \$${f}_owner/\$${f}..."
  [[ "$CERT" == "2" ]] && sslCAinfo="" || sslCAinfo="-c http.sslCAinfo=/etc/ssl/certs/ca-certificates.crt"

  rm -rf "$workdir"/data/repo/
  eval git clone -q $sslCAinfo -c remote.origin.proxy="" -c user.name="$GITEA_ADMIN_USER" -c user.email="$ADMIN_EMAIL" -c remote.origin.url=https://"$token"@git."$SERVICE_DOMAIN"/\$${f}_owner/\$${f}.git https://"$token"@git."$SERVICE_DOMAIN"/\$${f}_owner/\$${f}.git "$workdir"/data/repo/ || { echo -e "FAIL\nОшибка клонирования репозитория"; exit 1; }
  rsync -rtc --exclude='.git/' --exclude='.idea/' "$workdir"/data/distr/ "$workdir"/data/repo/
  cd "$workdir"/data/repo/ || exit 1
  git add . 1> /dev/null
  git commit -m "Автоматическое обновление из дистрибутива" 1> /dev/null
  git push origin main &> /dev/null && echo "ОК" || { echo -e "FAIL\nОшибка обновления репозитория"; exit 1; }
  cd "$workdir" || exit 1
  rm -rf "$workdir"/data/*
  
  # # обновляем данные в хранилище
  # eval "$installdir"/updater/sync.sh \$${f}_owner \$${f} main
done