#!/bin/bash

function usage {
  echo "Использование: webhook.sh [org1] ... [orgN]"
  echo "  org - организация (владелец) репозитория"
  echo -e "\nСкрипт предназначен для добавления webhook синхронизации хранилища."
  echo    "Если параметр org не указан, то добавляет системную webhook, но, в случае устаревшего API Gitea,"
  echo    "добавляет webhook во все или в указанные организации."
}

function set_org_webhook {
  if [[ -z ${orgs:-} ]]; then
    # получить список организаций
    while :; do
      if ! responseCode=$(\
        curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
          https://git.$SERVICE_DOMAIN/api/v1/orgs?token=$token \
          -H 'accept: application/json'); then
        echo -e "Ошибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 1; }
      fi
      break
    done
    [[ ! "$responseCode" == "200" ]] && { echo -e "Ошибка получения списка организаций\nКод: $responseCode\nСообщение: $(jq -j '.[].message' .apiresponse)"; exit 2; }
    mapfile -t orgs < <(jq -r '.[].name' .apiresponse)  
  fi
  
  # обрабатываем каждую организацию
  for org in "${orgs[@]}"; do
    # получаем хуки организации
    while :; do
      if ! responseCode=$(\
        curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
          https://git.$SERVICE_DOMAIN/api/v1/orgs/$org/hooks?token=$token \
          -H 'accept: application/json'); then
        echo -e "Ошибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 1; }
      fi
      break
    done
    [[ ! "$responseCode" == "200" ]] && { echo -e "Ошибка получения webhook в организации $org\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"; exit 2; }
    
    # пропускаем орг если хука имеется
    webhook=$(jq '.[] | select(.config.url=="http://updater:9000/hooks/gitea")' .apiresponse)
    [[ -z $webhook ]] || { echo "Webhook уже настроен в организации $org"; continue; }
        
    # создать хуку в организации
    while :; do
      if ! responseCode=$(\
        curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
        https://git.$SERVICE_DOMAIN/api/v1/orgs/$org/hooks?token=$token \
        -H 'accept: application/json'       \
        -H 'Content-Type: application/json' \
        -d '{
        "active": true,
        "config": {
          "url": "http://updater:9000/hooks/gitea",
          "content_type": "json"
        },
        "type": "gogs",
        "events": [
          "create",
          "delete",
          "fork",
          "push",
          "issues",
          "issue_assign",
          "issue_label",
          "issue_milestone",
          "issue_comment",
          "pull_request",
          "pull_request_assign",
          "pull_request_label",
          "pull_request_milestone",
          "pull_request_comment",
          "pull_request_review_approved",
          "pull_request_review_rejected",
          "pull_request_review_comment",
          "pull_request_sync",
          "wiki",
          "repository",
          "release",
          "package"
        ]
        }'); then
        echo -e "Ошибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 1; }
      fi
      if [[ "$responseCode" == "201" ]]; then
        echo -e "Webhook добавлен в организацию $org"
      else
        echo -e "Ошибка добавления webhook в организацию $org\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; }
      fi
      break
    done
  done
}

function set_system_webhook {
  echo "Проверяем наличие webhook для синхронизации хранилища"
  
  # Из-за нестабильности в Gitea v1.19.3 API работы со стандартными и системными вебхуками
  # (создаётся стандартный, а не системный webhook, невозможно получить список стандартных webhook), 
  # добавляем вебхуки в организации. Убрать этот блок после стабилизации API
  # и использовать системные вебхуки вместо вебхуков организаций. TODO
  # set_org_webhook
  # true && return

  while :; do
    if ! responseCode=$(\
      curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
        https://git.$SERVICE_DOMAIN/api/v1/admin/hooks?token=$token \
        -H 'accept: application/json'); then
      echo -e "Ошибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
      read -r -p "Повторить? (Y/n): " ANSWER
      [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 1; }
    fi
    break
  done
  if [ "$responseCode" = 404 ]; then
    # добавляем webhook в организацию
    set_org_webhook
  else
    # проверяем наличие стандартного webhook
    [[ "$responseCode" == "200" ]] || { echo -e "Ошибка получения стандартных webhook\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"; exit 2; }
    webhook=$(jq '.[] | select(.config.url=="http://updater:9000/hooks/gitea")' .apiresponse)
    [[ -z $webhook ]] || return
    
    # создать webhook
    echo "Добавляем стандартный webhook"
    while :; do
      if ! responseCode=$(\
        curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
        https://git.$SERVICE_DOMAIN/api/v1/admin/hooks?token=$token \
        -H 'accept: application/json'       \
        -H 'Content-Type: application/json' \
        -d '{
        "active": true,
        "config": {
          "url": "http://updater:9000/hooks/gitea",
          "content_type": "json"
        },
        "type": "gogs"
        }'); then
        echo -e "Ошибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 2; }
      fi
      if [[ "$responseCode" == "201" ]]; then
        echo -e "Webhook добавлен"
        break
      else
        echo -e "Ошибка добавления webhook\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 2; }
      fi
    done
  fi
}

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> "$(dirname "$(realpath -s "$0")")"/."$(basename "$0")".log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

workdir=$(dirname "$(realpath -s "$0")")
installdir=$(dirname "$workdir")
cd "$workdir" || exit 1

[ -f $installdir/.env ] || { echo "Отсутствует файл $installdir/.env. Выполните установку PromUC."; exit 2; }
source "$installdir"/.env
[[ -n $SERVICE_DOMAIN ]] || { echo "Выполните установку PromUC"; exit 2; }

# проверяем настройку Gitea
source "$installdir"/gitea/check.sh
check_token_exist

if [[ $# -eq 1 ]]; then
  case $1 in
    -h|--help) usage; exit ;;
  esac
fi
orgs=( "$@" )

# настраиваем стандартный webhook
set_system_webhook