#!/bin/bash


# DEBUG options
if [ -n "${DEBUG:-}" ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  PS4='$LINENO: '
  BASH_XTRACEFD="5"
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем Updater\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd "$workdir" || exit 1

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  
  [[ -f $installdir/.env ]] && source "$installdir"/.env
  cp template.env .env
  
  # from install/functions.sh
  set_service_domain

  UPDATER_IP=$(hostname -I | awk '{print $1}')
  sed -i -r "s|(UPDATER_IP=).*|\1$UPDATER_IP|g" .env
  
  source .env
  source "$installdir"/.env

  [ $(docker compose ps -aq | wc -l) -gt 0 ] && { echo "Останавливаем Updater"; docker compose down -v; }
  stop_storage_volume

  # проверяем настройку Gitea
  source "$installdir"/gitea/check.sh
  check_token_exist
  
else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки удалите файл $(pwd)/.env или используйте флаг --recreate\n"
fi

source $installdir/proxy/.env
if [[ ! "$CERT" == "2" ]]; then 
  [ -d ca-certificates.crt ] && rm -rf ca-certificates.crt
  if [ -f ../proxy/conf/certs/ca-certificates.crt ]; then
    cp -p ../proxy/conf/certs/ca-certificates.crt .
  else
    echo -e "Отсутствует файл сертификатов ca-certificates.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
  [ -d ca.crt ] && rm -rf ca.crt
  if [ -f ../proxy/conf/certs/ca.crt ]; then
    cp -p ../proxy/conf/certs/ca.crt .
  else
    echo -e "Отсутствует файл сертификатов ca.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
fi

echo -e "\nЗапускаем updater"
[ $(docker image ls -q psm/updater:latest | wc -l) -eq 0 ] && { "$installdir"/getimage.sh psm/updater:latest || exit 1; }
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 1; }

# останавливаем службу и процесс
if [ -f /usr/local/lib/systemd/system/promuc-updater.service ]; then
  while :; do
    echo -n "Останавливаем службу promuc-updater..."
    systemctl stop promuc-updater &>/dev/null && { echo "ОК"; break; }
    echo "FAIL"
    IFS= read -erp "Повторить? (Y/n): "
    [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2
  done
fi
ps --no-headers axk comm o pid,args | awk '$2 ~ "'"$workdir"'/webhook"{print $1}' | xargs -r kill -9
rm -f webhook

echo -n "Получаем файл Updater..."
docker cp -q updater:/app/webhook "$workdir" || { echo -e "FAIL\nОшибка копирования из контейнера updater"; exit 1; }
echo 'OK'
chmod +x "$workdir"/webhook

if [[ -z ${UPDATER_PORT:-} ]] || [ $(ss -ltn | grep -c "$UPDATER_IP:$UPDATER_PORT") -gt 0 ]; then
  echo "Сетевой порт для сервиса promuc-updater не указан либо занят другим процессом"
  while :; do
    read -erp "Укажите номер порта для сервиса promuc-updater: " UPDATER_PORT
    [ -z "$UPDATER_PORT" ] && continue
    [ "$UPDATER_PORT" = 0 ] && continue
    if [[ $(ss -ltn | grep -c "$UPDATER_IP:$UPDATER_PORT") -gt 0 ]]; then
      echo "Порт $UPDATER_PORT уже занят другим процессом"
      continue
    fi
    sed -i -r "s|(UPDATER_PORT=).*|\1$UPDATER_PORT|g" .env
    break
  done
fi

[ ! -f /usr/local/lib/systemd/system/promuc-updater.service ] && mkdir -p /usr/local/lib/systemd/system
cat > /usr/local/lib/systemd/system/promuc-updater.service <<EOF
[Unit]
Description=PromUC local Updater service
Documentation=https://promuc.ru/#doc
Before=docker.service
After=network.target iptables.service firewalld.service ip6tables.service ipset.service nftables.service

[Service]
ExecStart=$installdir/updater/webhook -hotreload -hooks $installdir/updater/host/hooks.yaml -verbose -ip $UPDATER_IP -port $UPDATER_PORT
ExecStartPost=$installdir/updater/host/start_plugins.sh
WorkingDirectory=$installdir/updater
    
[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
systemctl enable promuc-updater &>/dev/null
while :; do
  echo -n "Запускаем службу promuc-updater..."
  systemctl start promuc-updater &>/dev/null && { echo "ОК"; break; }
  echo "FAIL"
  IFS= read -erp "Повторить? (Y/n): "
  [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2
done

# синхронизируем репозитории
"$workdir"/sync.sh -a