#!/bin/bash

function usage {
  [[ ${1:-} = "-s" ]] || echo "Скрипт предназначен для выполнения синхронизации репозитория с хранилищем"
  echo "Использование: sync.sh -a | <owner> <repo> <branch>"
  echo "-a,--all - синхронизация всех репозиториев"
  echo "   owner - владелец репозитория"
  echo "    repo - название репозитория"
  echo "  branch - название ветки репозитория"
  exit 1
}

function valid_ip()
{
    local  ip=$1
    local  stat=1

    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

function prepare {
  # устанавливаем необходимые утилиты
  if [ -z "$(which jq)" ] || [ -z "$(which curl)" ]; then
    echo "Требуется установка утилит jq и curl, обновляем кеш APT..."
    { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo -e "FAIL\nВозможны ошибки при установке необходимых пакетов" || echo "OK"
    echo -n "Устанавливаем jq curl..."
    apt-get -qq install jq curl 1> /dev/null && echo "OK" || { echo "FAIL"; exit 1; }
  fi 

  [[ $(docker compose -f "$installdir"/updater/docker-compose.yml ps -q | wc -l) -gt 0 ]] || { echo -e "FAIL\nНе запущен сервис Updater"; exit 2; }
  # получаем и проверяем валидность IP адреса Updater
  updater_ip_addr=$(docker inspect -f '{{range.NetworkSettings.Networks}}{{.IPAddress}}{{end}}' updater)
  if ! valid_ip "$updater_ip_addr"; then
    echo -e "FAIL\nПолучен не верный IPv4 адрес контейнера Updater"
    exit 1
  fi
}

function sync {
  echo -n "Синхронизируем ветку $branch репозитория $owner/$repo с хранилищем..."
  while :; do
    if ! responseCode=$(\
      curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
        "http://$updater_ip_addr:9000/hooks/gitea" \
        -H 'Content-Type: application/json' \
        -H 'X-Gitea-Event-Type: push' \
        -d '{
          "ref": "'"$branch"'",
          "ref_type": "branch",
          "repository": {
            "name": "'"$repo"'",
            "owner": {
              "login": "'"$owner"'"
            }
          }      
        }'
    ); then
      echo -e "FAIL\nОшибка при обращении к API Updater\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
      read -r -p "Повторить? (Y/n): " ANSWER
      [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте работоспособность Updater и повторите импорт данных demo-scada или, для принудительной синхронизации хранилища, перейдите по ссылке https://$SERVICE_DOMAIN/api/hooks_gitea/promuc/demo-scada/main"; exit 2; }
    fi
    break
  done
  [[ "$responseCode" == "200" ]] && echo "OK" || { echo -e "FAIL\nПроверьте работоспособность Updater и повторите импорт данных demo-scada или, для принудительной синхронизации хранилища, перейдите по ссылке https://$SERVICE_DOMAIN/api/hooks_gitea/promuc/demo-scada/main"; exit 2; }
}

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> "$(dirname "$(realpath -s "$0")")"/."$(basename "$0")".log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

workdir=$(dirname "$(realpath -s "$0")")
installdir=$(dirname "$workdir")
cd "$workdir" || exit 1

if [[ $# -eq 1 ]]; then
  case $1 in
    -h|--help) usage ;;
    -a|--all) sync_all=true ;;
  esac
elif [[ $# -lt 3 ]]; then
  usage -s
fi

source "$installdir"/.env
prepare

if [[ ${sync_all:-false} = true ]]; then
  
  # проверяем настройку Gitea
  source "$installdir"/gitea/check.sh
  check_token_exist
  
  while :; do
    echo -n "Получаем список репозиториев..."
    if ! responseCode=$(\
      curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
        https://git.$SERVICE_DOMAIN/api/v1/repos/search?token=$token \
        -H 'accept: application/json'); then
      echo -e "FAIL\nОшибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
      read -r -p "Повторить? (Y/n): "
      [[ "${REPLY:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите синхронизацию"; exit 1; }
    fi
    break
  done
  if [[ ! "$responseCode" == "200" ]]; then
    { echo -e "FAIL\nОшибка получения списка репозиториев\nКод: $responseCode\nСообщение: $(jq -j '.[].message' .apiresponse)"; exit 2; }
  else
    mapfile -t repos < <(jq -r '.data[].full_name' .apiresponse)
    [[ ${#repos[@]} -gt 0 ]] && echo "OK" || { echo -e "FAIL\nНет репозиториев для синхронизации"; exit; }
  fi  
    
  for repo in "${repos[@]}"; do
    while :; do
      if ! responseCode=$(\
        curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
          https://git.$SERVICE_DOMAIN/api/v1/repos/$repo/branches?token=$token \
          -H 'accept: application/json'); then
        echo -e "FAIL\nОшибка при обращении к API Gitea\nКод: $responseCode\nСообщение: $(jq -j '.message' .apiresponse)"
        read -r -p "Повторить? (Y/n): "
        [[ "${REPLY:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите синхронизацию"; exit 1; }
      fi
      break
    done
    if [[ ! "$responseCode" == "200" ]]; then
      { echo -e "FAIL\nОшибка получения списка веток\nКод: $responseCode\nСообщение: $(jq -j '.[].message' .apiresponse)"; exit 2; }
    else
      mapfile -t branches < <(jq -r '.[].name' .apiresponse)
      [[ ${#branches[@]} -gt 0 ]] || { echo "Нет веток для синхронизации"; continue; }
    fi

    owner=$(echo "$repo" | sed -r 's/^(.+)\/.+$/\1/')
    repo=$(echo "$repo" | sed -r 's/^.+\/(.+)$/\1/')
    
    # readarray -t owner_repo < <(sed 's/\// /' <<< $repo)
    for branch in "${branches[@]}"; do
      sync
    done
  done
else
  owner=$1
  repo=$2
  branch=$3
  sync
fi


