#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi
set -uo pipefail

get_help() {
  echo "Скрипт обновления установки Promuc"
  echo -e "Запуск: upgrade.sh -h | -d | [-r] -f <filename>\n"
  echo -e "\t-f <filename> - файл дистрибутива для обновления"
  echo -e "\t-d - скачать дистрибутив для обновления"
  echo -e "\t-r - восстановление из указанного файла"
  echo -e "\t-h - вызов этой справки"
  exit
}

forwardRequest() {
  IFS= read -r -p "Продолжаем? (Y/n): " ANSWER
  [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || { rm -rf $tmpdir; exit 1; }
}

checkargs () {
  [ $1 = true ] && return
  if [[ $OPTARG =~ ^- ]]; then
    echo "Неверный аргумент $OPTARG для опции -$opt"
    get_help
    exit 1
  else
    file=$OPTARG
    if [[ $file == "." ]]  || \
       [[ $file == ".." ]] || \
       [ $(echo "${#file}") -gt 255 ] || \
       ! [[ $file =~ ^[/0-9a-zA-Z._-]+$ ]] || \
       ! [[ $(echo $file | cut -c1-1) =~ ^[/0-9a-zA-Z.]$ ]]; then
       echo "Ошибка в пути или имени файла, допустимые символы 0-9a-zA-Z/.-_"
       exit 1
    fi
  fi
  if [ ! -f $file ]; then
    echo "Файл $file не существует или нет доступа к нему"
    exit 1
  elif [[ -z $(file $file | grep "gzip compressed data") ]]; then
    echo "Указанный файл не соответствует типу tar.gz"
    exit 1
  fi
}

loadimg () {
  docker load -i $1 || { echo "Ошибка при загрузке docker-образа из файла $(readlink -f $1)"; forwardRequest; }
}

serviceDown() {
  echo -e "\nОстанавливаем и удаляем контейнеры сервисов...\n"
  $workdir/do.sh -r --all down
  if [ $($workdir/do.sh --all ps -q | grep -v "Run " | wc -w) -gt 0 ]; then
    echo -e "\nПроверьте удалены ли все контейнеры, при необходимости остановите и удалите.\n"
    rm -rf $tmpdir
    exit 1
  fi
}

recovery() {
  [ -z $file ] && { echo "Не указан файл для восстановления"; exit 1; }

  echo -e "\nБудет выполнено восстановление файлов из резервной копии $file в $workdir.\nКонтейнеры сервисов будут пересозданы."
  forwardRequest
  
  echo "Проверяем целостность архива..."
  files=$(tar -tvf $file | sed '/\/$/d' | awk '{ print $6 }')
  if [ $? -ne 0 ]; then
    echo "Ошибка проверки файла"
    exit 1
  fi
  [ $(echo $files | sed 's/ /\n/g' | grep -e "^upgrade.info$" | wc -l) -eq 0 ] && { echo "Выберите файл созданный при обновлении"; exit 1; }
  images=$(echo $files | sed 's/ /\n/g' | grep -e "^images/.*\.bz2$")
  files=$(echo $files | sed 's/ /\n/g' | sed '/\.bz2$/d;/^upgrade.info$/d')
  if [ -z "$files" ] && [ -z "$images" ]; then
    echo "Нет файлов и образов для восстановления"
    exit
  fi
  [ ! -z "$files" ] && echo -e "Будут восстановлены файлы:\n$files\n"
  [ ! -z "$images" ] && echo -e "\nБудут восстановлены docker-образы:\n$images\n"
  forwardRequest
  serviceDown
  echo -e "Восстанавливаем..."
  files=$(tar --exclude='upgrade.info' -xvf $file)
  if [ $? -eq 0 ]; then
    echo -e "Файлы восстановлены"
  else
    echo -e "\nОшибка восстановления\n"
    exit 1
  fi
  
  if [ ! -z $images ]; then
    echo "Загружаем восстановленные образы..."
    for image in $images; do
      loadimg $image
    done
  fi
  
  echo "Восстановление окончено. Запускаем контейнеры сервисов?"
  forwardRequest
  $workdir/do.sh --all up -d
  exit
}

workdir=$(echo $(dirname $(realpath -s $0)))
cd $workdir

file=""
download=false
recovery=false
tmpdir=""

[ "$#" -eq 0 ] && { echo "Не указаны опции"; get_help; }

while getopts ":hdf:r" opt; do
  case $opt in
    h ) get_help ;;
    f ) checkargs $download; file=$(realpath -s $OPTARG) ;;
    d ) download=true ;;
    : ) echo "Ошибка, не указан файл" 1>&2; get_help; exit 1 ;;
    r ) recovery=true ;;
    \? ) echo "Передан неверный аргумент: -$OPTARG" 1>&2; get_help; exit 1 ;;
  esac
done
shift $((OPTIND -1))

[ $recovery = true ] && recovery

echo -e "\nВнимание! данный скрипт заменит существующие установочные файлы в $workdir и пересоздаст контейнеры PromUC.\nБудет создана резервная копия заменяемых файлов в $workdir/install/.\nДля восстановления предыдущей версии используйте upgrade.sh -r -f <имя файла>.\n"

if [ -z $(which rsync) ] || [ -z $(which curl) ] || [ -z $(which bzip2) ]; then
  echo "Устанавливаем rsync curl bzip2"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install rsync curl bzip2 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

if [ $download = true ] && [ -z $file ]; then
  IFS= read -r -p "Скачиваем файл дистрибутива, без docker-образов (1) или полный (2): " ANSWER
  [ "$ANSWER" = "2" ] && filename="promuc-debian-full.tar.gz" || filename="promuc-debian.tar.gz"
  ddir=$(dirname $workdir)
  echo "Файл дистрибутива будет сохранён в $ddir/$filename"
  cd $ddir
  if [ -f $filename ]; then
    echo "Обнаружен файл дистрибутива $ddir/$filename, будет переименован в $ddir/$filename.old"
    mv $filename $filename.old
  fi
  echo "Получаем дистрибутив c адреса https://download.protei-sm.ru/$filename"
  curl -O -f -# --retry 3 --retry-delay 5 -Y 1 https://download.protei-sm.ru/$filename || { echo "Ошибка при получении файла дистрибутива, проверьте доступ к Интернет и адресу https://download.protei-sm.ru"; exit 1; }
  file="$ddir/$filename"
  cd $workdir  
fi

## Проверка версий
tmpdir="/tmp/upgrade/"
rm -rf $tmpdir
mkdir $tmpdir || { echo "Ошибка создания каталога $tmpdir"; exit 1; }
echo "Распаковываем дистрибутив в /tmp/upgrade"
tar -xf $file -C $tmpdir
if [[ $? -ne 0 ]]; then
  echo "Ошибка распаковки файла"
  rm -rf $tmpdir
  exit 1
fi  
cd $tmpdir/promuc || { echo "В файле дистрибутива отсутствует корневой каталог promuc"; rm -rf $tmpdir; exit 1; }
if [ ! -f version ]; then
  echo "В $file отсутствует необходимый файл version"
  rm -rf $tmpdir
  exit 1
fi

newV=$(cat ./version)
usedV=$(cat $workdir/version)
echo -e "Исходный файл содержит версию $(cat ./version), используемая версия $(cat $workdir/version)"
if [ "$newV" = "$usedV" ]; then
  echo "Версии идентичны. Проверим файлы..."
elif [ "$newV" = "$(echo -e "$newV\n$usedV" | sort -V | head -n1)" ]; then
  echo "Внимание! Используемая версия новее чем в файле"
  forwardRequest
fi

## Подготовка списка файлов
rsyncRepFiles=$(rsync -rtvcn --omit-dir-times --existing --exclude='images/' ./ $workdir/ | head -n-3 | tail -n+2)
rsyncAddFiles=$(rsync -rtvcn --omit-dir-times --ignore-existing --exclude='images/' ./ $workdir/ | grep -v "/$" | head -n-3 | tail -n+2)
if [ $(echo $rsyncRepFiles | wc -w) -eq 0 ]; then
  echo -e "Существующие файлы идентичны"
else
  echo -e "\nБудут заменены файлы:"
  echo $rsyncRepFiles | sed 's/ /\n/g'
fi
if [ $(echo $rsyncAddFiles | wc -w) -gt 0 ]; then
  echo -e "\nБудут добавлены файлы:"
  echo $rsyncAddFiles | sed 's/ /\n/g'
  if [  "$(echo "$rsyncAddFiles" | grep -c docker-compose.yml)" -gt 0 ]; then
    echo -e "\nВнимание! Новые сервисы:"
    for line in $(echo $rsyncAddFiles | grep "docker-compose.yml"); do
      echo dirname $line
    done
    echo -e "Если необходимо их добавить в состав комплекса, то после обновления:\n\t1) добавьте их в $workdir/.order согласно $workdir/template.order\n\t2) выполните настройку и запуск командой './install.sh --recreate' в каталоге нового сервиса"
  fi
fi
if [ -d ./images ]; then
  echo -e "\nПроверяем идентичность файлов docker-образов..."
  rsyncRepImg=$(rsync -rtvcn --omit-dir-times --existing images/ $workdir/images/ | head -n-3 | tail -n+2)
  rsyncAddImg=$(rsync -rtvcn --omit-dir-times --ignore-existing images/ $workdir/images/ | grep -v "/$" | head -n-3 | tail -n+2)
  if [ $(echo $rsyncRepImg | wc -w) -eq 0 ]; then
    echo "Не требуется замена файлов docker-образов"
  else
    echo -e "\nФайлы docker-образов с изменениями:"
    echo $rsyncRepImg | sed 's/ /\n/g'
  fi
  if [ $(echo $rsyncAddImg | wc -w) -gt 0 ]; then
    echo -e "\nБудут добавлены файлы docker-образов:"
    echo $rsyncAddImg | sed 's/ /\n/g'
  fi
else
  rsyncRepImg=""
  rsyncAddImg=""
fi

if [[ $(echo $rsyncRepFiles $rsyncAddFiles $rsyncRepImg $rsyncAddImg | wc -w) -eq 0 ]]; then
  echo "Обновление не требуется"
  exit
elif [[ $(echo $rsyncRepFiles $rsyncRepImg | wc -w) -eq 0 ]]; then
  echo "Т.к. заменяемые файлы отсутствуют архив не будет создан"
  backupdir=""
else
  backupdir="$workdir/install/upgrade_$(date +%y%m%d%H%M%S)"
  echo -e "\nРезервные копии заменяемых файлов будут сохранены в ${backupdir}.tar.gz\nДля их восстановления используйте команду $workdir/upgrade.sh -r ${backupdir}.tar.gz\n"
fi

# определяем необходимость остановки сервисов
svcdown=()
if [ $(echo $rsyncRepFiles | wc -w) -gt 0 ]; then
  for line in $rsyncRepFiles; do
    [[ ! $line =~ / ]] && continue
    svc=${line%%/*}
    [ -f $workdir/$svc/docker-compose.yml ] && svcdown+=( $svc )
  done
fi
if [ ${#svcdown[*]} -gt 0 ] || [ $(echo $rsyncRepImg | wc -w) -gt 0 ]; then
  echo -e "Внимание! Требуется предварительное удаление контейнеров сервисов, т.к. планируются изменения в файлах сервисов или в docker-образах."
  forwardRequest
  serviceDown
else
  forwardRequest
fi

## Синхронизация содержимого
echo -e "\nВыполняем обновление..."
if [ ! -z $backupdir ]; then
  rm -rf $backupdir
  mkdir $backupdir
  rsync -rtc -b --backup-dir=$backupdir ./ $workdir/
else
  rsync -rtc ./ $workdir/
fi
if [ $? -eq 0 ]; then
  echo -e "Синхронизация установочных файлов выполнена успешно"
  if [ ! -z $backupdir ]; then
    cd $backupdir || exit 1
    echo -e "Дата обновления:\t$(date --rfc-3339='seconds')\nФайл обновления:\t$file\nДата файла:\t$(date -r $file)\nshasum -a256:\t$(shasum -a256 $file | cut -d' ' -f1)\nВерсия дистрибутива:\t$newV\nОбновлённая версия:\t$usedV" >> upgrade.info
    echo "Упаковываем резервные копии файлов в архив ${backupdir}.tar.gz"
    tar -czpf ../${backupdir##*/}.tar.gz * && rm -rf $backupdir || echo -e "Ошибка при архивировании $backupdir в $backupdir.tar.gz"
  fi
else
  echo -e "\nОшибка синхронизации установочных файлов"
  if [ ! -z $backupdir ]; then
    echo "Восстанавливаем заменённые ранее файлы"
    rsync -rtvc $backupdir/ $workdir/ && echo "Выполнено успешно" || echo -e "Ошибка при возврате файлов из $backupdir в $workdir"
  fi
  rm -rf $tmpdir
  exit 1  
fi

cd $tmpdir/promuc
if [ ! -z $rsyncRepImg ] || [ ! -z $rsyncAddImg ]; then
  echo -e "\nЗагружаем обновлённые docker-образы...\n"
  for image in $rsyncAddImg; do
    echo "Загружаем $image"
    [[ $(file $workdir/images/$image | grep "bzip2 compressed data" | wc -l) -eq 0 ]] && continue
    loadimg $workdir/images/$image
  done
  for image in $rsyncRepImg; do
    [[ $(file $workdir/images/$image | grep "bzip2 compressed data" | wc -l) -eq 0 ]] && continue
    imagename=$(echo $image | sed -rn 's|(.*)\.bz2|\1|;s|__|:|;s|_|\/|p')
    imageid=$(docker image ls -q -f reference=$imagename | head -n 1)
    echo "Обновляем docker-образ $imagename"
    if [ -z $imageid ] || [[ $imagename =~ : ]]; then
      # образа нет
      loadimg $workdir/images/$image
    elif [[ ! $imagename =~ : ]]; then
      # образ есть и в названии не содержится версия
      echo "Уже имеется загруженный docker-образ $imagename:latest c id $imageid, проверяем даты создания..."
      imageprevdate=$(date -d $(docker image inspect $imageid -f '{{.Created}}') +%s)
      loadimg $workdir/images/$image
      loadedimgdate=$(date -d $(docker image inspect $imagename -f '{{.Created}}') +%s)
      if [ $imageprevdate -gt $loadedimgdate ]; then
        IFS= read -r -p "Внимание! В файле содержится образ $imagename предыдущей версии. Использовать его? (y/n): " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue
      fi
      docker tag $imageid $imagename:latest
      echo "Образ $imageid восстановлен в $imagename:latest"
      tar -C $workdir/ -xf $backupdir.tar.gz images/${image##*/} && echo "Файл docker-образа восстановлен в $workdir/images/$image" || echo "Ошибка при восстановлении images/${image##*/} из $backupdir.tar.gz"
    fi
  done
fi
rm -rf $tmpdir

echo -e "Обновление завершено. Запускаем контейнеры сервисов?"
forwardRequest
cd $workdir
./do.sh --all up -d
